/*
** Module   : NamedEventManager.java
** Abstract : provides a publish/subscribe mechanism for interprogram linkage
**
** Copyright (c) 2007, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ------------------Description-------------------
** 001 GES 20070221   @32210 Created initial version which provides a
**                           publish/subscribe mechanism for interprogram
**                           linkage.
** 002 CA  20130130          Added APIs to support all possible parameters
**                           for PUBLISH, SUBSCRIBE and UNSUBSCRIBE.
*/ 

package com.goldencode.p2j.util;

/**
 * Provides a publish/subscribe mechanism for interprogram linkage. This
 * corresponds to the following language statements in Progress 4GL. Actual
 * implementation of each statement resides in specific 
 * {@link ProcedureManager} APIs.
 * <p>
 * <pre>
 * PUBLISH                         {@link ProcedureManager#publish}
 * SUBSCRIBE                       {@link ProcedureManager#subscribe}
 *                                 {@link ProcedureManager#subscribeAnywhere}
 * UNSUBSCRIBE                     {@link ProcedureManager#unsubscribe}
 *                                 {@link ProcedureManager#unsubscribeAll}
 * </pre>
 *
 * @author    GES
 */
public class NamedEventManager
{
   /**
    * Notify all subscribers (to the given event) that the event has
    * occurred.
    * <p>
    * All processing is done with silent error mode implicitly enabled.
    *
    * @param    event
    *           The name of the event which is being published.
    * @param    publisher
    *           The procedure handle representing the program that is the
    *           source of the event. Must not be <code>null</code>.
    */
   public static void publish(String event, handle publisher)
   {
      publish(new character(event), publisher);
   }

   /**
    * Notify all subscribers (to the given event) that the event has
    * occurred.
    * <p>
    * All processing is done with silent error mode implicitly enabled.
    *
    * @param    event
    *           The name of the event which is being published.
    * @param    publisher
    *           The procedure handle representing the program that is the
    *           source of the event. Must not be <code>null</code>.
    */
   public static void publish(character event, handle publisher)
   {
      publish(event, publisher, null);
   }
   
   /**
    * Notify all subscribers (to the given event) that the event has
    * occurred.
    * <p>
    * All processing is done with silent error mode implicitly enabled.
    *
    * @param    event
    *           The name of the event which is being published.
    * @param    publisher
    *           The procedure handle representing the program that is the
    *           source of the event. Must not be <code>null</code>.
    * @param    param
    *           The parameters passed to the subscriber. May be
    *           <code>null</code> to represent no parameters.
    */
   public static void publish(String event, handle publisher, Object[] param)
   {
      publish(new character(event), publisher, param);
   }

   /**
    * Notify all subscribers (to the given event) that the event has
    * occurred.
    * <p>
    * All processing is done with silent error mode implicitly enabled.
    *
    * @param    event
    *           The name of the event which is being published.
    * @param    publisher
    *           The procedure handle representing the program that is the
    *           source of the event. Must not be <code>null</code>.
    * @param    param
    *           The parameters passed to the subscriber. May be
    *           <code>null</code> to represent no parameters.
    */
   public static void publish(character event,
                              handle    publisher,
                              Object[]  param)
   {
      ProcedureManager.publish(event, publisher, param);
   }
   
   /**
    * Create a new subscription to the given event for a specified program.
    * This program will receive a call-back when the event is published.
    *
    * @param    subscriber
    *           The program to notify when the specified event is published.
    *           Must not be <code>null</code>.  The specified program must
    *           have a method that corresponds to a Progress 4GL internal
    *           procedure name created from the given event name.
    * @param    event
    *           The name of the event which is being published.
    */
   public static void subscribe(handle subscriber, String event)
   {
      subscribe(subscriber, new character(event));
   }

   /**
    * Create a new subscription to the given event for a specified program.
    * This program will receive a call-back when the event is published.
    *
    * @param    subscriber
    *           The program to notify when the specified event is published.
    *           Must not be <code>null</code>.  The specified program must
    *           have a method that corresponds to a Progress 4GL internal
    *           procedure name created from the given event name.
    * @param    event
    *           The name of the event which is being published.
    */
   public static void subscribe(handle subscriber, character event)
   {
      subscribe(subscriber, event, null);
   }

   /**
    * Create a new subscription to the given event for a specified program.
    * This program will receive a call-back when the event is published.
    *
    * @param    subscriber
    *           The program to notify when the specified event is published.
    *           Must not be <code>null</code>.  The specified program must
    *           have a method that corresponds to a Progress 4GL internal
    *           procedure name created from the given event name.
    * @param    event
    *           The name of the event which is being published.
    * @param    publisher
    *           The procedure handle representing the program that is the
    *           source of the event.  If specified, the named event will only
    *           cause a notification if the event source matches this handle.
    *           <code>null</code> represents the <code>ANYWHERE</code> option
    *           which means that the events will not be filtered by source.
    */
   public static void subscribe(handle subscriber,
                                String event,
                                handle publisher)
   {
      subscribe(subscriber, new character(event), publisher);
   }

   /**
    * Create a new subscription to the given event for a specified program.
    * This program will receive a call-back when the event is published.
    *
    * @param    subscriber
    *           The program to notify when the specified event is published.
    *           Must not be <code>null</code>.  The specified program must
    *           have a method that corresponds to a Progress 4GL internal
    *           procedure name created from the given event name.
    * @param    event
    *           The name of the event which is being published.
    * @param    publisher
    *           The procedure handle representing the program that is the
    *           source of the event.  If specified, the named event will only
    *           cause a notification if the event source matches this handle.
    *           <code>null</code> represents the <code>ANYWHERE</code> option
    *           which means that the events will not be filtered by source.
    */
   public static void subscribe(handle    subscriber,
                                character event,
                                handle    publisher)
   {
      subscribe(subscriber, event, publisher, (character) null);
   }

   /**
    * Create a new subscription to the given event for a specified program.
    * This program will receive a call-back when the event is published.
    *
    * @param    subscriber
    *           The program to notify when the specified event is published.
    *           Must not be <code>null</code>.  The specified program must
    *           have a method that corresponds to the given Progress 4GL
    *           internal procedure name (<code>method</code>).
    * @param    event
    *           The name of the event which is being published.
    * @param    publisher
    *           The procedure handle representing the program that is the
    *           source of the event.  If specified, the named event will only
    *           cause a notification if the event source matches this handle.
    *           <code>null</code> represents the <code>ANYWHERE</code> option
    *           which means that the events will not be filtered by source.
    * @param    method
    *           The Progress 4GL internal procedure name which will be
    *           translated into a method name and invoked on the 
    *           <code>subscriber</code> instance. 
    */
   public static void subscribe(handle subscriber,
                                String event,
                                handle publisher,
                                String method)
   {
      subscribe(subscriber,
                new character(event), 
                publisher,
                new character(method));
   }

   /**
    * Create a new subscription to the given event for a specified program.
    * This program will receive a call-back when the event is published.
    *
    * @param    subscriber
    *           The program to notify when the specified event is published.
    *           Must not be <code>null</code>.  The specified program must
    *           have a method that corresponds to the given Progress 4GL
    *           internal procedure name (<code>method</code>).
    * @param    event
    *           The name of the event which is being published.
    * @param    publisher
    *           The procedure handle representing the program that is the
    *           source of the event.  If specified, the named event will only
    *           cause a notification if the event source matches this handle.
    *           <code>null</code> represents the <code>ANYWHERE</code> option
    *           which means that the events will not be filtered by source.
    * @param    method
    *           The Progress 4GL internal procedure name which will be
    *           translated into a method name and invoked on the 
    *           <code>subscriber</code> instance. 
    */
   public static void subscribe(handle    subscriber,
                                String    event,
                                handle    publisher,
                                character method)
   {
      subscribe(subscriber, new character(event), publisher, method);
   }

   /**
    * Create a new subscription to the given event for a specified program.
    * This program will receive a call-back when the event is published.
    *
    * @param    subscriber
    *           The program to notify when the specified event is published.
    *           Must not be <code>null</code>.  The specified program must
    *           have a method that corresponds to the given Progress 4GL
    *           internal procedure name (<code>method</code>).
    * @param    event
    *           The name of the event which is being published.
    * @param    publisher
    *           The procedure handle representing the program that is the
    *           source of the event.  If specified, the named event will only
    *           cause a notification if the event source matches this handle.
    *           <code>null</code> represents the <code>ANYWHERE</code> option
    *           which means that the events will not be filtered by source.
    * @param    method
    *           The Progress 4GL internal procedure name which will be
    *           translated into a method name and invoked on the 
    *           <code>subscriber</code> instance. 
    */
   public static void subscribe(handle    subscriber,
                                character event,
                                handle    publisher,
                                String    method)
   {
      subscribe(subscriber, event, publisher, new character(method));
   }
   
   /**
    * Create a new subscription to the given event for a specified program.
    * This program will receive a call-back when the event is published.
    *
    * @param    subscriber
    *           The program to notify when the specified event is published.
    *           Must not be <code>null</code>.  The specified program must
    *           have a method that corresponds to the given Progress 4GL
    *           internal procedure name (<code>method</code>).
    * @param    event
    *           The name of the event which is being published.
    * @param    publisher
    *           The procedure handle representing the program that is the
    *           source of the event.  If specified, the named event will only
    *           cause a notification if the event source matches this handle.
    *           <code>null</code> represents the <code>ANYWHERE</code> option
    *           which means that the events will not be filtered by source.
    * @param    method
    *           The Progress 4GL internal procedure name which will be
    *           translated into a method name and invoked on the 
    *           <code>subscriber</code> instance. 
    */
   public static void subscribe(handle    subscriber,
                                character event,
                                handle    publisher,
                                character method)
   {
      ProcedureManager.subscribe(subscriber, event, publisher, method);
   }
   
   /**
    * Create a new subscription to the given event for any program.
    * This program will receive a call-back when the event is published.
    *
    * @param    subscriber
    *           The program to notify when the specified event is published.
    *           Must not be <code>null</code>.  The specified program must
    *           have a method that corresponds to the given Progress 4GL
    *           internal procedure name (<code>method</code>).
    * @param    event
    *           The name of the event which is being published.
    */
   public static void subscribeAnywhere(handle subscriber, String event)
   {
      subscribeAnywhere(subscriber, new character(event));
   }

   /**
    * Create a new subscription to the given event for any program.
    * This program will receive a call-back when the event is published.
    *
    * @param    subscriber
    *           The program to notify when the specified event is published.
    *           Must not be <code>null</code>.  The specified program must
    *           have a method that corresponds to the given Progress 4GL
    *           internal procedure name (<code>method</code>).
    * @param    event
    *           The name of the event which is being published.
    */
   public static void subscribeAnywhere(handle subscriber, character event)
   {
      subscribeAnywhere(subscriber, event, (character) null);
   }

   /**
    * Create a new subscription to the given event for any program.
    * This program will receive a call-back when the event is published.
    *
    * @param    subscriber
    *           The program to notify when the specified event is published.
    *           Must not be <code>null</code>.  The specified program must
    *           have a method that corresponds to the given Progress 4GL
    *           internal procedure name (<code>method</code>).
    * @param    event
    *           The name of the event which is being published.
    * @param    method
    *           The Progress 4GL internal procedure name which will be
    *           translated into a method name and invoked on the 
    *           <code>subscriber</code> instance. 
    */
   public static void subscribeAnywhere(handle subscriber,
                                        String event,
                                        String method)
   {
      subscribeAnywhere(subscriber,
                        new character(event),
                        new character(method));
   }

   /**
    * Create a new subscription to the given event for any program.
    * This program will receive a call-back when the event is published.
    *
    * @param    subscriber
    *           The program to notify when the specified event is published.
    *           Must not be <code>null</code>.  The specified program must
    *           have a method that corresponds to the given Progress 4GL
    *           internal procedure name (<code>method</code>).
    * @param    event
    *           The name of the event which is being published.
    * @param    method
    *           The Progress 4GL internal procedure name which will be
    *           translated into a method name and invoked on the 
    *           <code>subscriber</code> instance. 
    */
   public static void subscribeAnywhere(handle    subscriber,
                                        character event,
                                        String    method)
   {
      subscribeAnywhere(subscriber, event, new character(method));
   }

   /**
    * Create a new subscription to the given event for any program.
    * This program will receive a call-back when the event is published.
    *
    * @param    subscriber
    *           The program to notify when the specified event is published.
    *           Must not be <code>null</code>.  The specified program must
    *           have a method that corresponds to the given Progress 4GL
    *           internal procedure name (<code>method</code>).
    * @param    event
    *           The name of the event which is being published.
    * @param    method
    *           The Progress 4GL internal procedure name which will be
    *           translated into a method name and invoked on the 
    *           <code>subscriber</code> instance. 
    */
   public static void subscribeAnywhere(handle    subscriber,
                                        String    event,
                                        character method)
   {
      subscribeAnywhere(subscriber, new character(event), method);
   }

   /**
    * Create a new subscription to the given event for any program.
    * This program will receive a call-back when the event is published.
    *
    * @param    subscriber
    *           The program to notify when the specified event is published.
    *           Must not be <code>null</code>.  The specified program must
    *           have a method that corresponds to the given Progress 4GL
    *           internal procedure name (<code>method</code>).
    * @param    event
    *           The name of the event which is being published.
    * @param    method
    *           The Progress 4GL internal procedure name which will be
    *           translated into a method name and invoked on the 
    *           <code>subscriber</code> instance. 
    */
   public static void subscribeAnywhere(handle    subscriber,
                                        character event,
                                        character method)
   {
      ProcedureManager.subscribeAnywhere(subscriber, event, method);
   }

   /**
    * Delete a subscription to the given event for the specified program.
    *
    * @param    subscriber
    *           The program whose subscription is to be deleted.  Must not
    *           be <code>null</code>.
    * @param    event
    *           The name of the event which is being published. If this is
    *           <code>null</code>, then subscriptions to all events will be
    *           deleted.
    */
   public static void unsubscribe(handle subscriber, String event)
   {
      unsubscribe(subscriber, new character(event));
   }
   
   /**
    * Delete a subscription to the given event for the specified program.
    *
    * @param    subscriber
    *           The program whose subscription is to be deleted.  Must not
    *           be <code>null</code>.
    * @param    event
    *           The name of the event which is being published. If this is
    *           <code>null</code>, then subscriptions to all events will be
    *           deleted.
    */
   public static void unsubscribe(handle subscriber, character event)
   {
      unsubscribe(subscriber, event, null);
   }
   
   /**
    * Delete a subscription to the given event for the specified program.
    *
    * @param    subscriber
    *           The program whose subscription is to be deleted.  Must not
    *           be <code>null</code>.
    * @param    event
    *           The name of the event which is being published. If this is
    *           <code>null</code>, then subscriptions to all events will be
    *           deleted.
    * @param    publisher
    *           The procedure handle representing the program that is the
    *           source of the event.  If specified, the subscription will only
    *           be deleted if the event source matches this handle.
    *           <code>null</code> represents the <code>ALL</code> option
    *           which means that the events will not be filtered by source.
    */
   public static void unsubscribe(handle subscriber, 
                                  String event,
                                  handle publisher)
   {
      unsubscribe(subscriber, new character(event), publisher);
   }
   
   /**
    * Delete a subscription to the given event for the specified program.
    *
    * @param    subscriber
    *           The program whose subscription is to be deleted.  Must not
    *           be <code>null</code>.
    * @param    event
    *           The name of the event which is being published. If this is
    *           <code>null</code>, then subscriptions to all events will be
    *           deleted.
    * @param    publisher
    *           The procedure handle representing the program that is the
    *           source of the event.  If specified, the subscription will only
    *           be deleted if the event source matches this handle.
    *           <code>null</code> represents the <code>ALL</code> option
    *           which means that the events will not be filtered by source.
    */
   public static void unsubscribe(handle    subscriber,
                                  character event,
                                  handle    publisher)
   {
      ProcedureManager.unsubscribe(subscriber, event, publisher);
   }

   /**
    * Delete a subscription to ALL events for the specified program.
    *
    * @param    subscriber
    *           The program whose subscription is to be deleted.  Must not
    *           be <code>null</code>.
    */
   public static void unsubscribeAll(handle subscriber)
   {
      unsubscribeAll(subscriber, null);
   }

   /**
    * Delete a subscription to ALL events for the specified program.
    *
    * @param    subscriber
    *           The program whose subscription is to be deleted.  Must not
    *           be <code>null</code>.
    * @param    publisher
    *           The procedure handle representing the program that is the
    *           source of the event.  If specified, the subscription will only
    *           be deleted if the event source matches this handle.
    *           <code>null</code> represents the <code>ALL</code> option
    *           which means that the events will not be filtered by source.
    */
   public static void unsubscribeAll(handle subscriber, handle publisher)
   {
      ProcedureManager.unsubscribeAll(subscriber, publisher);
   }
}
